%
% DummyLectures.pl (Automatische Stundenplangenerierung)
%
% Die Prdikate um die Dummy-Vorlesungen der Datenbasis-Constraints vorlesungsfreieTage/2
% sowie fr die Mittagspausen "Vorlesungen" der fr die Hrer-Gruppen zu erstellen.
%
% Um Dummy-Rume von regulren Rumen unterscheiden zu knnen bekommen ALLE regulren Rume und Vorlesungen
% die Resource 'regular'.
% dummyVorlesung/1(Vorlesungsname) -> Vorlesungen die als dummyLecture/1 definiert sind, werden zwar 
%                           whrend der Stundenplanung bercksichtigt, tauchen aber nicht im Stundenplan auf,
%			    (sie finden ohnehin in nichtregulren Rumen statt !)

/**********************************************************************/
%
% prepareDummyLectures/0 
%
% Erstellt die beiden Rume mensa sowie freizeit, die jeweils eine 
% groesse von 3000 Hrern haben, und erstellt die DummyVorlesungen 
% fr die freien Tage von Dozenten sowie die Mittagspausen von 
% Hrergruppen. Die entsprechenden Fakten werden in die Datenbasis
% geschrieben.
%
/**********************************************************************/
prepareDummyLectures :-
    /* Prdikate der DB die modifiziert werden sollen als DYNAMIC definieren 
    dynamic vorlesungAm/2,vorlesungNichtUm/2,vorlesungNichtAm/2,raum/3,
    dynamic vorlesung/4,dummyVorlesung/1,allBlocksSameDay/1,mittagszeit/1,    */

    asserta(raum(mensa,   [groesse(3000),mensa]   ,100)),
    asserta(raum(freizeit,[groesse(3000),freizeit],100)),
    asserta(dummyRaum(mensa)),
    asserta(dummyRaum(freizeit)),
    
    noMiddayTimes(NoMiddayTimes),
    asserta(getNoMiddayTimes(NoMiddayTimes)),
    
    findall(Gruppenname,(group(Gruppenname,_),not(dummyGroup(Gruppenname))),Gruppennamen),
    mkMiddays(Gruppennamen),
    findall(Dozent,dozent(Dozent),Dozenten),
    mkFreeDays(Dozenten).
    
/**********************************************************************/
%
% mkFreeDays/1(+DozList)
%
% Erstellt fr jeden Dozenten in der Liste DozList die Dummy-Vorlesungen
% die sich aus der Definition mit vorlesungsfreieTage/2 fr den Dozenten
% ergibt. Sind fr einen Dozenten keine Vorlesungsfreien Tage definiert,
% werden die entsprechenden Dummy-Vorlesungen nicht angelegt.
% Benutzt mkFreeDaysForLecturer/2 um Vorlesungsfreie Tage fr einen 
% Dozenten zu erstellen. Die entsprechenden Fakten werden in die 
% Datenbasis geschrieben.
%
% +DozList Liste von Dozenten fr die geprft werden soll ob vorlesungsfreie
%          Tage eingehalten werden mssen.
%
/**********************************************************************/
mkFreeDays([]).
mkFreeDays([Dozent|T]) :-
    vorlesungsfreieTage(Dozent,N),
    N>0,!,
    mkFreeDaysForLecturer(Dozent,N),
    mkFreeDays(T).
mkFreeDays([_|T]) :-
    mkFreeDays(T).
    

/**********************************************************************/
%
% mkFreeDaysForLecturer/2(+Dozent,+AnzTage)
%
% Erstellt fr den Angegebenen Dozenten +Dozent Dummy-Vorlesungen, so da er
% +AnzTage in der Woche frei hat. Die entsprechenden Fakten werden in die 
% Datenbasis geschrieben
%
/**********************************************************************/
mkFreeDaysForLecturer(_,0).
mkFreeDaysForLecturer(Dozent,N):-
    M is N-1,
    mkFreeDaysDorLecturer(Dozent,M),
    noOfTimes(NOfTimes),
    asserta(vorlesung(['Frei',Dozent,N],[Dozent],[freizeit],[single(NOfTimes)])),
    asserta(dummyVorlesung(['Frei',Dozent,N])),
    asserta(alleBloeckeAmSelbenTag(['Frei',Dozent,N])).

    
/**********************************************************************/
%
% mkMiddays/1(+Groups)
%
% Erstellt fr jede Gruppe aus der Liste +Groups fr jeden Tag eine
% Dummyvorlesung, die im Raum mensa zur Mittagszeit stattfindet.
%
% +Groups Liste von Gruppennamen
%
/**********************************************************************/
mkMiddays([]).
mkMiddays([GName|T]):-
    mkMiddaysForGroup(GName),
    mkMiddays(T).
    
mkMiddaysForGroup(GName) :-
    dayNumbers(Days),
    mkMiddaysForGroup(GName,Days).

mkMiddaysForGroup(_,[]).
mkMiddaysForGroup(GName,[nn(Day,_)|T]):-
    tagOhneMittag(Day),!,
    mkMiddaysForGroup(GName,T).
mkMiddaysForGroup(GName,[nn(Day,DayID)|T]):-
    mkMiddaysForGroup(GName,T),
    getNoMiddayTimes(ZeitenListe),
    asserta(vorlesung(['Mittag',GName,DayID],[GName],[mensa],[single(1)])),
    asserta(dummyVorlesung(['Mittag',GName,DayID])),
    asserta(vorlesungAm(['Mittag',GName,DayID],Day)),
    asserta(vorlesungNichtUm(['Mittag',GName,DayID],ZeitenListe)).
    
%
% Success wenn Tag keine Mittagszeit hat.
% 
tagOhneMittag(Tag) :-
    tageOhneMittag(Tage),
    member(Tag,Tage).

%
% Wenn +TimeName (Name, keine ID) keine Zeit zur Mittagszeit ist -> sucess.
%
keinMittag(TimeName):-
    mittagszeit(Zeiten),
    member(TimeName,Zeiten),!,fail.
keinMittag(_).

%
% Alle Zeiten die nicht zur Mittagszeit gehren.
%
noMiddayTimes(Zeiten) :-
    timeNumbers(Times),
    findall(Zeit,(member(nn(Zeit,_),Times),keinMittag(Zeit)),Zeiten).
    