%
% HTMLOutput.pl (Stundenplangenerierung)
%
% Erstellt einen leeren HTML-Stundenplan (Skeleton), dessen freie Felder 
% ber benannte freie Variable ausgefllt werden knnen.
%

/******************************************************/
/* Ausgabe fr allgemeine Skeletons                   */
/******************************************************/

/************************************************************************/
%
% writeOutListToStream/2(+Stream,+List)
%
% Schreibt die Elemente der Liste +List in den Stream +Stream, wobei 
% verschachtelte Listen rekursiv ebenfalls ausgegeben werden. Ungebundene
% Variable werden ignoriert. Wird benutzt um (ausgefllte) Skeletons auszugeben.
%
% +Stream Hierhinein wird die Liste ausgegeben
% +List   Die auszugebende Liste
%
/************************************************************************/
writeOutListToStream(_,[]).
writeOutListToStream(Stream,[nl|T]) :-
    !,nl(Stream),
    writeOutListToStream(Stream,T).
writeOutListToStream(Stream,[H|T]) :-
    H=[_|_],!,
    writeOutListToStream(Stream,H),
    writeOutListToStream(Stream,T).    
writeOutListToStream(Stream,[OutListHeader|T]) :-
    var(OutListHeader),!,
    writeOutListToStream(Stream,T).
writeOutListToStream(Stream,[OutListHeader|T]) :-
    write(Stream,OutListHeader),
    writeOutListToStream(Stream,T).    

/***************************************************/
/*   HTML Skeleton                                 */
/***************************************************/

/************************************************************************/
%
% getOutSkeleton/5(+Type,-OutList,-HoleList,+Title,+Vars)
%
% Erstellt einen leeren Stundenplan -Outlist, dessen freie Felder ber eine
% Liste benannter freier Variable -HoleList ausgefllt werden knnen.  
% Der Titel des Planes ist +Title der Stundenplan fr den das Skeleton 
% erstellt werden soll wird durch die dtr/4-Variablenliste +Vars dargestellt.
% +Type gibt an welches Format der Stundenplan hat (z.b.: html)
%
% +Type Format des Stundenplanes. Momentan nur html mglich.
% -OutList Liste von hole/2(Name,Var) Strukturen. Wobei Name den Namen der freien
%          Variable Var darstellt. Als Namen kommen vor dtr/3(Tag,Zeit,Raum) fr
%          das freie Feld "Raum zur Zeit Zeit am Tag Tag" (Tag,Zeit,Raum sind dabei
%          gebunden an Indizies fr Tage Zeiten bzw. Rume) sowie grp/1(BID) mit dem
%          der freie Platz zur Eintragung von Gruppen fr den Block mit der BlockID BID
%          gekennzeichnet wird.
% -HoleList Liste von hole/2-Strukturen. Siehe -OutList.
% +Title   Der Titel des Stundenplanes.
% +Vars    Die dtr/4-Variablenliste, die das Ergebnis des Suchprozesses (also einen Stundenplan)
%          darstellt.
%
/************************************************************************/
getOutSkeleton(html,OutList,HoleList,Title,Vars) :-
   getHTMLOutSkeleton(OutList,HoleList,Title,Vars).
   
getHTMLOutSkeleton(Skeleton,Holes,Title,Vars) :-
    getHTMLSkeleton(Skeleton,HTMLHoles),
    fillIn(HTMLHoles,'TITLE',Title),
    getHTMLTimeTable(Table,Holes,Vars),
    fillIn(HTMLHoles,'TABLE',Table).

fillIn(Holes,Key,Value) :-
    member(hole(Key,Value),Holes),!.
fillIn(_,_,_). /* Immer erfolgreich */
    
getHTMLSkeleton(Skeleton,Holes) :-
    Skeleton=['<HTML>',nl,'<HEAD>',nl,'<TITLE>',nl,Title,nl,'</TITLE>',nl,
              '</HEAD>',nl,'<BODY bgcolor=#FFFFFF>',nl,Table,nl,'</BODY>',nl,'</HTML>',nl],
    Holes=[hole('TITLE',Title),hole('TABLE',Table)].

getHTMLTimeTable(Table,Holes,Vars) :-
    getHTMLTableHeaders(Header),
    Table=['<TABLE border=1 bgcolor=#FFFFE0 cellpadding=1 cellspacing=0>',nl,  /* Klammert die gesamte Tabelle    */
           '<TR bgcolor=#C0C0C0><TH><font size=3>Tag</font></TH><TH><font size=3>HS</font></TH>',                      /* Die Spaltenberschriften Tag,HS */
           Header,'</TR>',nl,Body,nl                                           /* -- " -- fr die Tageszeiten     */   
	   ,'</TABLE>'],
    noDummyRoomNumbers(Raeume),
    dayNumbers(Tage),
    timeNumbers(Stunden),
    htmlProcessAllDays(Tage,Stunden,Raeume,Body,Holes,Vars).

noDummyRoomNumbers(Raeume) :-
    roomNumbers(Rooms),
    findall(nn(RName,RNum),(member(nn(RName,RNum),Rooms),not(dummyRaum(RName))),Raeume).

getHTMLTableHeaders(Header) :-
    timeNumbers(Times),
    getHTMLTableHeaders(Times,Header).
    
getHTMLTableHeaders([],[]).
getHTMLTableHeaders([nn(TimeName,_)|T],Header) :-
    getHTMLTableHeaders(T,Header0),
    append(['<TH width=150><font size=3>',TimeName,'</font></TH>'],Header0,Header).
    
htmlProcessAllDays(Tage,Stunden,Raeume,Ergebnis,Holes,Vars) :-
    asserta(processedBID(dummy)),
    retract_all(processedBID(_)),
    htmlProcessDays(Tage,Stunden,Raeume,Ergebnis,Holes,Vars),
    retract_all(processedBID(_)).

htmlProcessDays([],_,_,[],[],_).
htmlProcessDays([nn(TagName,TagID)|Tage],Stunden,Raeume,Ergebnis,Holes,Vars) :-
    getHTMLDay(Table,DayHoles),
    fillIn(DayHoles,'DAY_NAME',TagName),  
    htmlProcessRooms(nn(TagName,TagID),Raeume,Stunden,Result0,RoomHoles,Vars),
    Result0=[_|Result],
    fillIn(DayHoles,'TABLE_BODY',Result),
    htmlProcessDays(Tage,Stunden,Raeume,Ergebnis0,Holes0,Vars),
    append(Table,Ergebnis0,Ergebnis),
    append(RoomHoles,Holes0,Holes).

getHTMLDay(Table,Holes):-
    noOfRooms(NOfRooms),  noOfTimes(NOfTimes), noOfDummyRooms(NOfDummyRooms),
    RowSpan is NOfRooms-NOfDummyRooms  ,  ColSpan is NOfTimes+2,
    Table=['<TR><TD rowspan=',RowSpan,' align=center bgcolor=#808080><font size=3><B>',DayName,
           '</B></font></TD>',nl,Body,nl,'</TR><TR height=0><TD colspan=',ColSpan,'></TD></TR>',nl],
    Holes=[hole('DAY_NAME',DayName),hole('TABLE_BODY',Body)].

htmlProcessRooms(_,[],_,[],[],_).
htmlProcessRooms(Tag,[nn(RaumName,RaumID)|Raeume],Stunden,Result,Holes,Vars):-
    htmlProcessRooms(Tag,Raeume,Stunden,Result0,Holes0,Vars),
    StartRow=['<TR>','<TD align=center bgcolor=#C0C0C0 height=30><font size=2><b>',RaumName,'</b></font></TD>'],
    htmlProcessTimes(Tag,nn(RaumName,RaumID),Stunden,Times,Holes1,Vars),
    append(StartRow,Times,ThisRow0), append(ThisRow0,['</TR>'],ThisRow),
    append(ThisRow,Result0,Result),
    append(Holes0,Holes1,Holes).
    

htmlProcessTimes(_,_,[],[],[],_).
htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),[nn(_,StdID)|Stunden],Result,Holes,Vars) :-
    alreadyProcessedBID(TagID,StdID,RaumID,Vars),!,
    htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),Stunden,Result,Holes,Vars).    
htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),[nn(_,StdID)|Stunden],Result,Holes,Vars) :-
    member(dtr(TagID,StdID,RaumID,BID),Vars),!,
    asserta(processedBID(BID)),
    htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),Stunden,Result0,Holes0,Vars),
    blockSize(BID,BlockSize),
    Width is BlockSize*150,
    append(['   <TD colspan=',BlockSize,'><Table width=',Width,
            ' cellpadding=1 cellspacing=0><TR><TD align=left valign=top><font size=2>',DayTimeRoom,
	    '</font></TD></TR><TR><TD align=right valign=bottom><font size=2>',Gruppen,
	    '</font></TD></TR></Table></TD>',nl],Result0,Result),
    append(Holes0,[hole(dtr(TagID,StdID,RaumID),DayTimeRoom),hole(grp(BID),Gruppen)],Holes).    
htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),[nn(_,_)|Stunden],Result,Holes,Vars) :-
    htmlProcessTimes(nn(TagName,TagID),nn(RaumName,RaumID),Stunden,Result0,Holes,Vars),     
    append(['   <TD bgcolor=#F0D306>&nbsp;</TD>',nl],Result0,Result).


alreadyProcessedBID(Day,Time,Room,Vars) :-
    member(dtr(Day,Time,Room,BID),Vars),!,
    processedBID(BID).


